clear; clc
% options:
    % ALGORITHM
    % choose feature selection algorithm. 1 == exhaustive, 2 == forward
    % sequential
    algorithm = 2;
    
        % TOOLBOX FORWARD SEQUENTIAL
        % if algorithm == 2:
        % choose whether to use the open source forward sequential  
        % algo or the one from the MATLAB Statistics and
        % Machine Learning toolbox
        useMLToolboxSeq = true;

    % NUMFEATURES
    % maximum number of features to be selected.
    numWantedFeatures = 2;
    
    % ECOC: CODING DESIGN
    % choose the coding design. available designs: 'onevsone', 'onevsall',
    % 'ordinal'
    coding = 'ordinal';
    
    % ECOC: ORDINAL ORDER
    % for the ordinal coding design to work, you have to specify the
    % ordinal order of the categories. Gain this information from
    % analyzing the plot. Only effective if coding = 'ordinal'
    
    %ordinalOrder = {'sparkling water';'cola';'apple juice';'wine'};
    ordinalOrder = {'sparkling water';'cola';'apple juice';'wine'};
    
        % TOOLBOX ECOC
        % use ECOC from Toolbox?
        % open source ECOC implementation is yet TO-DO so leave this = true!!
        useMLToolboxECOC = true;
    
    % STANDARDIZATION
    % standardize data before training? usually leads to better results
    standardize = true;
        

%------------------------
addpath('lib');
load("allTemperatures30-400.mat")

 if ~exist('ordinalOrder','var')
        ordinalOrder = categories(Ycategories);
 end

% standardize data
 if standardize
     Xarray = table2array(X);
     Xarray = (Xarray - mean(Xarray)) ./ std(Xarray);
     X = array2table(Xarray);
 end
 
% execute the appropiate algorithm
switch algorithm
    case 1
        selectedFeatures = exhaustiveSearch(X,Ycategories, numWantedFeatures,...
            coding, ordinalOrder);
    case 2
        selectedFeatures = sequentialSearch(X,Ycategories, numWantedFeatures,...
            coding, useMLToolboxSeq, ordinalOrder);
end

% plot results
createPlotsUpTo3D(X,Ycategories,selectedFeatures);

% save selected features
save('selectedFeatures.mat', 'selectedFeatures');
