function featureVector = exhaustiveSearch(X, Ycategories, numWantedFeatures,...
    coding, ordinalOrder)
%EXHAUSTIVESEARCH performs exhaustive search feature selection 
%   on data in table X, with
%   each row of X being a sample and each column being a feature, and
%   categorical vector Ycategories containing the targets.
%   numWantedFeatures: maximum number of features chosen.
%   coding: chosen coding design
%   useMLToolboxECOC: boolean indicating what training function to use
%
%   Warning: This algorithm is exponential in time, with numFeatures being
%   the exponent.
global combinations

numFeatures = size(X,2);
X = table2array(X);
bestPerformance = 0; % worst performance = max loss
featureVector = [];

disp("Exhaustive Search");
combinations={};
disp("Calculating all possible subsets. This may take a while.");
for i=1:numWantedFeatures
    combinations = [combinations;...
        nchoosek(1:numFeatures,i)];
end

numCombinations = 0;
for i=1:length(combinations)
    numCombinations = numCombinations + size(combinations{i},1);
end

for i = 1:numCombinations
    currentCombination = getCurrentCombination();
    disp(['Training step ' int2str(i) '/' int2str(numCombinations) ': '...
        'Features:']);
    disp(currentCombination);
        
        Mdl = executeECOC_SVMTraining(X(:,currentCombination),...
        Ycategories, coding, ordinalOrder);
        performance = 1 - loss(Mdl, X(:,currentCombination),...
            Ycategories);
        disp("Performance: ")
        disp(performance);
        if performance > bestPerformance
            featureVector = currentCombination;
            bestPerformance = performance;
        end
end
getCurrentCombination(1);

end

function result = getCurrentCombination(clear)
    global combinations
    persistent featuresCount
    persistent combinationCount
    if isempty(featuresCount) || exist('clear','var')
        featuresCount = 1;
    end
    if isempty(combinationCount) || exist('clear','var')
        combinationCount = 1;
    end
    if exist('clear','var')
        return
    end
    
    if length(combinations) >= featuresCount
        curFeatureCell = combinations{featuresCount};
        if size(curFeatureCell,1) >= combinationCount
            result = curFeatureCell(combinationCount,:);
            combinationCount = combinationCount + 1;
        else
           featuresCount = featuresCount + 1;
           combinationCount = 1;
           result = getCurrentCombination();
        end
    else
        result = [];
    end   
end

